unit Polygon;

(*********************************************
TPolygonSprite->TSprite

This descendant of TSprite renders itself by drawing
a set of lines.  This is an abstract class ... to
implement, derive a new class and override the
AddVertices method, which must add elements to the
class's lstVertices TList.

To create, call the CreatePolygonSprite constructor,
passing the color of the object (which can be used in
the overriden AddVertices) and an instance of a
TLookupTable that exists on the form.
*********************************************)

interface

uses
  SysUtils, WinTypes, WinProcs, Classes, Graphics, Controls, WinGPnt, Sprite,
  LkupTabl;

type

  TRotation = ( rotNone, rotClockwise, rotCounterClockwise );

  TVertex = class( TObject )
  private
  protected
  public
     ptPoint: TPoint;
     nCol: byte;
     constructor Create( pt: TPoint; nColor: byte );
  end;

  TPolygonSprite = class( TSprite )
  private
  protected
     lk: TLookupTable;
     lstVertices: TList;
     lstPreXformed: TList;
     nAngle: integer;
     procedure AddVertices( const nCol: byte ); virtual; abstract;
     procedure Render( winGPnt: TWinGPaintBox ); override;
     procedure Rotate( nAngle: integer );
     procedure SetAngle( n: integer );
  public
     Spin: TRotation;
     SpinSpeed: integer;
     property Angle: integer read nAngle write SetAngle;
     constructor CreatePolygonSprite( const nCol: byte; lkup: TLookupTable );
     destructor Destroy; override;
  end;

  TSquareSprite = class( TPolygonSprite )
  private
  protected
     procedure AddVertices( const nCol: byte ); override;
  public
  end;

implementation

constructor TVertex.Create( pt: TPoint; nColor: byte );
begin
  ptPoint := pt;
  nCol := nColor;
end;

constructor TPolygonSprite.CreatePolygonSprite( const nCol: byte; lkup: TLookupTable );
var
  i: integer;
  vert: TVertex;
begin
  inherited Create;
  lk := lkup;
  Spin := rotNone;
  SpinSpeed := 10;
  Angle := 0;
  lstVertices := TList.Create;
  lstPreXformed := TList.Create;
  AddVertices( nCol );
  for i := 0 to lstVertices.Count - 1 do
     begin
        vert := TVertex( lstVertices[i] );
        lstPreXformed.Add( TVertex.Create( vert.ptPoint, vert.nCol ) );
     end;
end;

destructor TPolygonSprite.Destroy;
var
  i: integer;
begin
  for i := 0 to lstVertices.Count - 1 do
     begin
        TVertex( lstVertices[i] ).Free;
        TVertex( lstPreXformed[i] ).Free;
     end;
  lstVertices.Clear;
  lstVertices.Free;
  lstPreXformed.Clear;
  lstPreXformed.Free;
  Spin := rotNone;
  inherited Destroy;
end;

(*********************************************
Spinning of the polygon is not considered important,
so it is done within render so it will not take
up processing time if the sprite is off the edge
of the surface.
*********************************************)
procedure TPolygonSprite.Render( winGPnt: TWinGPaintBox );
var
  i: integer;
  ptFrom, ptTo: TPoint;

  procedure AdjustPoints;
  begin
     Inc( ptFrom.X, ptPhysical.X );
     Inc( ptFrom.Y, ptPhysical.Y );
     Inc( ptTo.X, ptPhysical.X );
     Inc( ptTo.Y, ptPhysical.Y );
  end;

begin
  case Spin of
     rotClockwise:
        Angle := Angle + SpinSpeed;
     rotCounterClockwise:
        Angle := Angle - SpinSpeed;
  end;

  Rotate( 360 - ( Angle - 90 ) );

  for i := 1 to lstVertices.Count - 1 do
     begin
        ptFrom := TVertex( lstVertices[i - 1] ).ptPoint;
        ptTo := TVertex( lstVertices[i] ).ptPoint;
        AdjustPoints;
        winGPnt.Line( ptFrom.X, ptFrom.Y, ptTo.X, ptTo.Y, TVertex( lstVertices[i - 1] ).nCol );
     end;

  ptFrom := TVertex( lstVertices[lstVertices.Count - 1] ).ptPoint;
  ptTo := TVertex( lstVertices[0] ).ptPoint;
  AdjustPoints;

  WinGPnt.Line( ptFrom.X, ptFrom.Y, ptTo.X, ptTo.Y, TVertex( lstVertices[i - 1] ).nCol );
end;

procedure TPolygonSprite.Rotate( nAngle: integer );
var
  xSin, xCos: single;
  i: integer;
  vert, vertPre: TVertex;
begin
  if nAngle > 360 then
     begin
        xSin := lk.lkupSin[nAngle - 360];
        xCos := lk.lkupCos[nAngle - 360];
     end
  else if nAngle >= 0 then
     begin
        xSin := lk.lkupSin[nAngle];
        xCos := lk.lkupCos[nAngle];
     end
  else
     begin
        xSin := lk.lkupSin[nAngle + 360];
        xCos := lk.lkupCos[nAngle + 360];
     end;
  for i := 0 to lstVertices.Count - 1 do
     begin
        vert := TVertex( lstVertices[i] );
        vertPre := TVertex( lstPreXFormed[i] );
        begin
           vert.ptPoint.X := Round( vertPre.ptPoint.X * xCos - vertPre.ptPoint.Y * xSin );
           vert.ptPoint.Y := Round( vertPre.ptPoint.Y * xCos + vertPre.ptPoint.X * xSin );
        end;
     end;
end;

(*********************************************
Make sure angle remains within 0..360.
*********************************************)
procedure TPolygonSprite.SetAngle( n: integer );
begin
  nAngle := n;
  while nAngle < 0 do
     Inc( nAngle, 360 );
  while nAngle > 360 do
     Dec( nAngle, 360 );
end;

(*********************************************
TSquareSprite
An example descendant of TPolygonSprite.
*********************************************)
procedure TSquareSprite.AddVertices( const nCol: byte );
begin
  lstVertices.Add( TVertex.Create( Point( -5, -5 ), nCol ) );
  lstVertices.Add( TVertex.Create( Point( 5, -5 ), nCol ) );
  lstVertices.Add( TVertex.Create( Point( 5, 5 ), nCol ) );
  lstVertices.Add( TVertex.Create( Point( -5, 5 ), nCol ) );
  Spin := rotClockwise;
end;

end.
